<?php

namespace Drupal\entity_router;

use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\entity_router\Event\EntityResponseEvent;
use Drupal\entity_router\Response\EntityResponse;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * The event subscriber.
 */
class EventSubscriber implements EventSubscriberInterface {

  /**
   * Constructs the subscriber.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   An instance of the `event_dispatcher` service.
   * @param \Drupal\entity_router\EntityResponseHandlerManager $entityResponseHandlerManager
   *   An instance of the `*.plugin.manager.event_response_handler` service.
   */
  public function __construct(
    protected readonly EventDispatcherInterface $eventDispatcher,
    protected readonly EntityResponseHandlerManager $entityResponseHandlerManager,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [];

    $events[KernelEvents::RESPONSE][] = ['onResponse', 200];

    return $events;
  }

  /**
   * {@inheritdoc}
   */
  public function onResponse(ResponseEvent $event): void {
    $response = $event->getResponse();

    if ($response instanceof EntityResponse) {
      $entity = $response->entity;
      $request = $event->getRequest();
      // The Drupal's `_format` is forbidden by JSON:API 2.x.
      $format = $request->get('format', 'html');

      try {
        // Transform the response.
        $response = $this->entityResponseHandlerManager
          ->createInstance($format)
          ->getResponse($request, $entity)
          // Set actual status code before dispatching an event.
          ->setStatusCode($response->getStatusCode());

        if ($response instanceof CacheableResponseInterface) {
          $response
            ->addCacheableDependency($entity)
            ->addCacheableDependency((new CacheableMetadata())
              ->setCacheContexts([
                'url.query_args:path',
                'url.query_args:format',
                'url.query_args:include',
              ])
            );
        }

        $this->eventDispatcher->dispatch(new EntityResponseEvent($request, $response, $entity), EntityResponseEvent::NAME);
      }
      catch (PluginException $e) {
        $status = 400;
        $response = new Response(\sprintf('<h1>%d Bad Request.</h1><p>The "%s" request format cannot be handled.</p>', $status, $format), $status);
      }

      $event->setResponse($response);
    }
  }

}
